/**
 * Create a proxy of `tree` that acts like the original tree upon reading, but
 * each proxied node has a reference to its parent node.
 *
 * ###### Notes
 *
 * The returned proxy imposes two additional fields on all of its nodes:
 *
 * *   `parent` — parent link (or `undefined` for the root)
 * *   `node` — link to the original node
 *
 * These new fields are not enumerable and the original tree is *not changed*.
 * This means you can use `JSON.stringify` on the wrapped tree and it’s the
 * same.
 *
 * `wrapped.children` returns array of wrapped child nodes, so that any
 * recursive algorithm will work on a wrapped tree just as well.
 *
 * To write changes to the tree, use `.node` to access the original tree.
 *
 * @param {Node} tree
 *   Tree to proxy.
 * @returns {Proxy}
 *   Proxy of `tree`.
 */
export function parents(tree: Node): Proxy
export type Node = import('unist').Node
export type Parent = import('unist').Parent
/**
 * Extra fields found in a proxied node.
 */
export type ProxyFields = {
  /**
   *   Parent link (or `undefined` for the root).
   */
  parent: Parent | undefined
  /**
   *   Link to the original node
   */
  node: Node
}
/**
 * Proxied node.
 */
export type Proxy = Node & ProxyFields
